@@ -39,12 +39,13 @@ Datum		get_covers(PG_FUNCTION_ARGS);
 
 static float weights[] = {0.1, 0.2, 0.4, 1.0};
 
-#define wpos(wep)	( w[ ((WordEntryPos*)(wep))->weight ] )
+#define wpos(wep)	( w[ WEP_GETWEIGHT(wep) ] )
 
 #define DEF_NORM_METHOD 0
 
 static float calc_rank_or(float *w, tsvector * t, QUERYTYPE * q);
 static float calc_rank_and(float *w, tsvector * t, QUERYTYPE * q);
+
 /*
  * Returns a weight of a word collocation
  */
@@ -115,44 +116,52 @@ find_wordentry(tsvector * t, QUERYTYPE * q, ITEM * item)
 }
 
 
-static char * SortAndUniqOperand=NULL;
+static char *SortAndUniqOperand = NULL;
 
 static int
-compareITEM( const void * a, const void * b ) {
-	if (  (*(ITEM**)a)->length == (*(ITEM**)b)->length )
-		return strncmp( SortAndUniqOperand + (*(ITEM**)a)->distance,
-				SortAndUniqOperand + (*(ITEM**)b)->distance,
-				(*(ITEM**)b)->length );
+compareITEM(const void *a, const void *b)
+{
+	if ((*(ITEM **) a)->length == (*(ITEM **) b)->length)
+		return strncmp(SortAndUniqOperand + (*(ITEM **) a)->distance,
+					   SortAndUniqOperand + (*(ITEM **) b)->distance,
+					   (*(ITEM **) b)->length);
 
-	return ((*(ITEM**)a)->length > (*(ITEM**)b)->length) ? 1 : -1;
+	return ((*(ITEM **) a)->length > (*(ITEM **) b)->length) ? 1 : -1;
 }
-         
-static ITEM**
-SortAndUniqItems( char *operand, ITEM *item, int *size ) {
-	ITEM   **res, **ptr, **prevptr;
 
-	ptr = res = (ITEM**) palloc( sizeof(ITEM*) * *size );
+static ITEM **
+SortAndUniqItems(char *operand, ITEM * item, int *size)
+{
+	ITEM	  **res,
+			  **ptr,
+			  **prevptr;
 
-	while( (*size)-- ) {
-		if ( item->type == VAL ) {
+	ptr = res = (ITEM **) palloc(sizeof(ITEM *) * *size);
+
+	while ((*size)--)
+	{
+		if (item->type == VAL)
+		{
 			*ptr = item;
 			ptr++;
-		}   
+		}
 		item++;
 	}
 
-	*size = ptr-res;
-	if ( *size < 2 )
+	*size = ptr - res;
+	if (*size < 2)
 		return res;
 
-	SortAndUniqOperand=operand;
-	qsort( res, *size, sizeof(ITEM**), compareITEM );
+	SortAndUniqOperand = operand;
+	qsort(res, *size, sizeof(ITEM **), compareITEM);
 
 	ptr = res + 1;
 	prevptr = res;
 
-	while( ptr - res < *size ) {
-		if ( compareITEM( (void*) ptr, (void*) prevptr ) != 0 ) {
+	while (ptr - res < *size)
+	{
+		if (compareITEM((void *) ptr, (void *) prevptr) != 0)
+		{
 			prevptr++;
 			*prevptr = *ptr;
 		}
@@ -164,8 +173,8 @@ SortAndUniqItems( char *operand, ITEM *item, int *size ) {
 }
 
 static WordEntryPos POSNULL[] = {
-	{0, 0},
-	{0, MAXENTRYPOS - 1}
+	0,
+	0
 };
 
 static float
@@ -183,18 +192,19 @@ calc_rank_and(float *w, tsvector * t, QUERYTYPE * q)
 				lenct,
 				dist;
 	float		res = -1.0;
-	ITEM	   **item;
-	int size = q->size;
+	ITEM	  **item;
+	int			size = q->size;
 
-	item = SortAndUniqItems( GETOPERAND(q), GETQUERY(q), &size);
-	if ( size < 2 ) {
+	item = SortAndUniqItems(GETOPERAND(q), GETQUERY(q), &size);
+	if (size < 2)
+	{
 		pfree(item);
 		return calc_rank_or(w, t, q);
 	}
-
 	pos = (uint16 **) palloc(sizeof(uint16 *) * q->size);
-	memset(pos, 0, sizeof(uint16 **) * q->size);
+	memset(pos, 0, sizeof(uint16 *) * q->size);
 	*(uint16 *) POSNULL = lengthof(POSNULL) - 1;
+	WEP_SETPOS(POSNULL[1], MAXENTRYPOS - 1);
 
 	for (i = 0; i < size; i++)
 	{
@@ -220,14 +230,14 @@ calc_rank_and(float *w, tsvector * t, QUERYTYPE * q)
 			{
 				for (p = 0; p < lenct; p++)
 				{
-					dist = Abs(post[l].pos - ct[p].pos);
+					dist = Abs((int) WEP_GETPOS(post[l]) - (int) WEP_GETPOS(ct[p]));
 					if (dist || (dist == 0 && (pos[i] == (uint16 *) POSNULL || pos[k] == (uint16 *) POSNULL)))
 					{
 						float		curw;
 
 						if (!dist)
 							dist = MAXENTRYPOS;
-						curw = sqrt(wpos(&(post[l])) * wpos(&(ct[p])) * word_distance(dist));
+						curw = sqrt(wpos(post[l]) * wpos(ct[p]) * word_distance(dist));
 						res = (res < 0) ? curw : 1.0 - (1.0 - res) * (1.0 - curw);
 					}
 				}
@@ -247,15 +257,19 @@ calc_rank_or(float *w, tsvector * t, QUERYTYPE * q)
 	int4		dimt,
 				j,
 				i;
-	float		res = -1.0;
-	ITEM	   **item;
-	int	size = q->size;
+	float		res = 0.0;
+	ITEM	  **item;
+	int			size = q->size;
 
 	*(uint16 *) POSNULL = lengthof(POSNULL) - 1;
-	item = SortAndUniqItems( GETOPERAND(q), GETQUERY(q), &size);
+	item = SortAndUniqItems(GETOPERAND(q), GETQUERY(q), &size);
 
 	for (i = 0; i < size; i++)
 	{
+		float		resj,
+					wjm;
+		int4		jm;
+
 		entry = find_wordentry(t, q, item[i]);
 		if (!entry)
 			continue;
@@ -271,15 +285,30 @@ calc_rank_or(float *w, tsvector * t, QUERYTYPE * q)
 			post = POSNULL + 1;
 		}
 
+		resj = 0.0;
+		wjm = -1.0;
+		jm = 0;
 		for (j = 0; j < dimt; j++)
 		{
-			if (res < 0)
-				res = wpos(&(post[j]));
-			else
-				res = 1.0 - (1.0 - res) * (1.0 - wpos(&(post[j])));
+			resj = resj + wpos(post[j]) / ((j + 1) * (j + 1));
+			if (wpos(post[j]) > wjm)
+			{
+				wjm = wpos(post[j]);
+				jm = j;
+			}
 		}
+/*
+		limit (sum(i/i^2),i->inf) = pi^2/6
+		resj = sum(wi/i^2),i=1,noccurence,
+		wi - should be sorted desc,
+		don't sort for now, just choose maximum weight. This should be corrected
+		Oleg Bartunov
+*/
+		res = res + (wjm + resj - wjm / ((jm + 1) * (jm + 1))) / 1.64493406685;
 	}
-	pfree( item );
+	if (size > 0)
+		res = res / size;
+	pfree(item);
 	return res;
 }
 
@@ -288,6 +317,7 @@ calc_rank(float *w, tsvector * t, QUERYTYPE * q, int4 method)
 {
 	ITEM	   *item = GETQUERY(q);
 	float		res = 0.0;
+	int			len;
 
 	if (!t->size || !q->size)
 		return 0.0;
@@ -303,10 +333,12 @@ calc_rank(float *w, tsvector * t, QUERYTYPE * q, int4 method)
 		case 0:
 			break;
 		case 1:
-			res /= log((float) cnt_length(t));
+			res /= log((float) (cnt_length(t) + 1)) / log(2.0);
 			break;
 		case 2:
-			res /= (float) cnt_length(t);
+			len = cnt_length(t);
+			if (len > 0)
+				res /= (float) len;
 			break;
 		default:
 			/* internal error */
@@ -404,7 +436,7 @@ checkcondition_DR(void *checkval, ITEM * val)
 
 	while (ptr - ((ChkDocR *) checkval)->doc < ((ChkDocR *) checkval)->len)
 	{
-		if ( val == ptr->item || compareITEM( &val, &(ptr->item) ) == 0 )
+		if (val == ptr->item || compareITEM(&val, &(ptr->item)) == 0)
 			return true;
 		ptr++;
 	}
@@ -494,12 +526,11 @@ Cover(DocRepresentation * doc, int len, QUERYTYPE * query, int *pos, int *p, int
 		ch.doc = f;
 		ch.len = (doc + lastpos) - f + 1;
 		*pos = f - doc + 1;
-		SortAndUniqOperand = GETOPERAND(query); 
+		SortAndUniqOperand = GETOPERAND(query);
 		if (TS_execute(GETQUERY(query), &ch, false, checkcondition_DR))
 		{
 			/*
-			 * elog(NOTICE,"OP:%d NP:%d P:%d Q:%d", *pos, lastpos, *p,
-			 * *q);
+			 * elog(NOTICE,"OP:%d NP:%d P:%d Q:%d", *pos, lastpos, *p, *q);
 			 */
 			return true;
 		}
@@ -554,7 +585,7 @@ get_docrep(tsvector * txt, QUERYTYPE * query, int *doclen)
 		for (j = 0; j < dimt; j++)
 		{
 			doc[cur].item = &(item[i]);
-			doc[cur].pos = post[j].pos;
+			doc[cur].pos = WEP_GETPOS(post[j]);
 			cur++;
 		}
 	}
@@ -609,10 +640,12 @@ rank_cd(PG_FUNCTION_ARGS)
 		case 0:
 			break;
 		case 1:
-			res /= log((float) cnt_length(txt));
+			res /= log((float) (cnt_length(txt) + 1));
 			break;
 		case 2:
-			res /= (float) cnt_length(txt);
+			len = cnt_length(txt);
+			if (len > 0)
+				res /= (float) len;
 			break;
 		default:
 			/* internal error */
@@ -635,7 +668,7 @@ rank_cd_def(PG_FUNCTION_ARGS)
 										Int32GetDatum(-1),
 										PG_GETARG_DATUM(0),
 										PG_GETARG_DATUM(1),
-										(PG_NARGS() == 3) ? PG_GETARG_DATUM(2) : Int32GetDatum(DEF_NORM_METHOD)
+	  (PG_NARGS() == 3) ? PG_GETARG_DATUM(2) : Int32GetDatum(DEF_NORM_METHOD)
 										));
 }
 
@@ -713,7 +746,7 @@ get_covers(PG_FUNCTION_ARGS)
 		{
 			dw[cur].w = STRPTR(txt) + pptr[i].pos;
 			dw[cur].len = pptr[i].len;
-			dw[cur].pos = posdata[j].pos;
+			dw[cur].pos = WEP_GETPOS(posdata[j]);
 			cur++;
 		}
 		len += (pptr[i].len + 1) * (int) POSDATALEN(txt, &(pptr[i]));
