@@ -40,12 +40,28 @@
  *	  is used to run finalize functions and compute the output tuple;
  *	  this context can be reset once per output tuple.
  *
+ *	  Beginning in PostgreSQL 8.1, the executor's AggState node is passed as
+ *	  the fmgr "context" value in all transfunc and finalfunc calls.  It is
+ *	  not really intended that the transition functions will look into the
+ *	  AggState node, but they can use code like
+ *			if (fcinfo->context && IsA(fcinfo->context, AggState))
+ *	  to verify that they are being called by nodeAgg.c and not as ordinary
+ *	  SQL functions.  The main reason a transition function might want to know
+ *	  that is that it can avoid palloc'ing a fixed-size pass-by-ref transition
+ *	  value on every call: it can instead just scribble on and return its left
+ *	  input.  Ordinarily it is completely forbidden for functions to modify
+ *	  pass-by-ref inputs, but in the aggregate case we know the left input is
+ *	  either the initial transition value or a previous function result, and
+ *	  in either case its value need not be preserved.  See int8inc() for an
+ *	  example.	Notice that advance_transition_function() is coded to avoid a
+ *	  data copy step when the previous transition value pointer is returned.
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ *
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/executor/nodeAgg.c,v 1.116.2.3 2005/01/27 23:43:16 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/executor/nodeAgg.c,v 1.135.2.2 2007/02/02 00:07:44 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -67,6 +83,7 @@
 #include "utils/acl.h"
 #include "utils/builtins.h"
 #include "utils/lsyscache.h"
+#include "utils/memutils.h"
 #include "utils/syscache.h"
 #include "utils/tuplesort.h"
 #include "utils/datum.h"
@@ -92,8 +109,8 @@ typedef struct AggStatePerAggData
 
 	/*
 	 * fmgr lookup data for transfer functions --- only valid when
-	 * corresponding oid is not InvalidOid.  Note in particular that
-	 * fn_strict flags are kept here.
+	 * corresponding oid is not InvalidOid.  Note in particular that fn_strict
+	 * flags are kept here.
 	 */
 	FmgrInfo	transfn;
 	FmgrInfo	finalfn;
@@ -107,8 +124,8 @@ typedef struct AggStatePerAggData
 	Oid			sortOperator;
 
 	/*
-	 * fmgr lookup data for input type's equality operator --- only
-	 * set/used when aggregate has DISTINCT flag.
+	 * fmgr lookup data for input type's equality operator --- only set/used
+	 * when aggregate has DISTINCT flag.
 	 */
 	FmgrInfo	equalfn;
 
@@ -130,14 +147,14 @@ typedef struct AggStatePerAggData
 				transtypeByVal;
 
 	/*
-	 * These values are working state that is initialized at the start of
-	 * an input tuple group and updated for each input tuple.
+	 * These values are working state that is initialized at the start of an
+	 * input tuple group and updated for each input tuple.
 	 *
 	 * For a simple (non DISTINCT) aggregate, we just feed the input values
 	 * straight to the transition function.  If it's DISTINCT, we pass the
-	 * input values into a Tuplesort object; then at completion of the
-	 * input tuple group, we scan the sorted values, eliminate duplicates,
-	 * and run the transition function on the rest.
+	 * input values into a Tuplesort object; then at completion of the input
+	 * tuple group, we scan the sorted values, eliminate duplicates, and run
+	 * the transition function on the rest.
 	 */
 
 	Tuplesortstate *sortstate;	/* sort object, if a DISTINCT agg */
@@ -167,12 +184,11 @@ typedef struct AggStatePerGroupData
 	bool		noTransValue;	/* true if transValue not set yet */
 
 	/*
-	 * Note: noTransValue initially has the same value as
-	 * transValueIsNull, and if true both are cleared to false at the same
-	 * time.  They are not the same though: if transfn later returns a
-	 * NULL, we want to keep that NULL and not auto-replace it with a
-	 * later input value. Only the first non-NULL input will be
-	 * auto-substituted.
+	 * Note: noTransValue initially has the same value as transValueIsNull,
+	 * and if true both are cleared to false at the same time.	They are not
+	 * the same though: if transfn later returns a NULL, we want to keep that
+	 * NULL and not auto-replace it with a later input value. Only the first
+	 * non-NULL input will be auto-substituted.
 	 */
 } AggStatePerGroupData;
 
@@ -249,7 +265,7 @@ initialize_aggregates(AggState *aggstate,
 			peraggstate->sortstate =
 				tuplesort_begin_datum(peraggstate->inputType,
 									  peraggstate->sortOperator,
-									  false);
+									  work_mem, false);
 		}
 
 		/*
@@ -278,18 +294,18 @@ initialize_aggregates(AggState *aggstate,
 
 			oldContext = MemoryContextSwitchTo(aggstate->aggcontext);
 			pergroupstate->transValue = datumCopy(peraggstate->initValue,
-											 peraggstate->transtypeByVal,
-											  peraggstate->transtypeLen);
+												  peraggstate->transtypeByVal,
+												  peraggstate->transtypeLen);
 			MemoryContextSwitchTo(oldContext);
 		}
 		pergroupstate->transValueIsNull = peraggstate->initValueIsNull;
 
 		/*
-		 * If the initial value for the transition state doesn't exist in
-		 * the pg_aggregate table then we will let the first non-NULL
-		 * value returned from the outer procNode become the initial
-		 * value. (This is useful for aggregates like max() and min().)
-		 * The noTransValue flag signals that we still need to do this.
+		 * If the initial value for the transition state doesn't exist in the
+		 * pg_aggregate table then we will let the first non-NULL value
+		 * returned from the outer procNode become the initial value. (This is
+		 * useful for aggregates like max() and min().) The noTransValue flag
+		 * signals that we still need to do this.
 		 */
 		pergroupstate->noTransValue = peraggstate->initValueIsNull;
 	}
@@ -320,20 +336,18 @@ advance_transition_function(AggState *aggstate,
 		if (pergroupstate->noTransValue)
 		{
 			/*
-			 * transValue has not been initialized. This is the first
-			 * non-NULL input value. We use it as the initial value for
-			 * transValue. (We already checked that the agg's input type
-			 * is binary-compatible with its transtype, so straight copy
-			 * here is OK.)
+			 * transValue has not been initialized. This is the first non-NULL
+			 * input value. We use it as the initial value for transValue. (We
+			 * already checked that the agg's input type is binary-compatible
+			 * with its transtype, so straight copy here is OK.)
 			 *
-			 * We must copy the datum into aggcontext if it is pass-by-ref.
-			 * We do not need to pfree the old transValue, since it's
-			 * NULL.
+			 * We must copy the datum into aggcontext if it is pass-by-ref. We
+			 * do not need to pfree the old transValue, since it's NULL.
 			 */
 			oldContext = MemoryContextSwitchTo(aggstate->aggcontext);
 			pergroupstate->transValue = datumCopy(newVal,
-											 peraggstate->transtypeByVal,
-											  peraggstate->transtypeLen);
+												  peraggstate->transtypeByVal,
+												  peraggstate->transtypeLen);
 			pergroupstate->transValueIsNull = false;
 			pergroupstate->noTransValue = false;
 			MemoryContextSwitchTo(oldContext);
@@ -343,10 +357,9 @@ advance_transition_function(AggState *aggstate,
 		{
 			/*
 			 * Don't call a strict function with NULL inputs.  Note it is
-			 * possible to get here despite the above tests, if the
-			 * transfn is strict *and* returned a NULL on a prior cycle.
-			 * If that happens we will propagate the NULL all the way to
-			 * the end.
+			 * possible to get here despite the above tests, if the transfn is
+			 * strict *and* returned a NULL on a prior cycle. If that happens
+			 * we will propagate the NULL all the way to the end.
 			 */
 			return;
 		}
@@ -357,18 +370,9 @@ advance_transition_function(AggState *aggstate,
 
 	/*
 	 * OK to call the transition function
-	 *
-	 * This is heavily-used code, so manually zero just the necessary fields
-	 * instead of using MemSet().  Compare FunctionCall2().
 	 */
-
-	/* MemSet(&fcinfo, 0, sizeof(fcinfo)); */
-	fcinfo.context = NULL;
-	fcinfo.resultinfo = NULL;
-	fcinfo.isnull = false;
-
-	fcinfo.flinfo = &peraggstate->transfn;
-	fcinfo.nargs = 2;
+	InitFunctionCallInfoData(fcinfo, &(peraggstate->transfn), 2,
+							 (void *) aggstate, NULL);
 	fcinfo.arg[0] = pergroupstate->transValue;
 	fcinfo.argnull[0] = pergroupstate->transValueIsNull;
 	fcinfo.arg[1] = newVal;
@@ -377,12 +381,12 @@ advance_transition_function(AggState *aggstate,
 	newVal = FunctionCallInvoke(&fcinfo);
 
 	/*
-	 * If pass-by-ref datatype, must copy the new value into aggcontext
-	 * and pfree the prior transValue.	But if transfn returned a pointer
-	 * to its first input, we don't need to do anything.
+	 * If pass-by-ref datatype, must copy the new value into aggcontext and
+	 * pfree the prior transValue.	But if transfn returned a pointer to its
+	 * first input, we don't need to do anything.
 	 */
 	if (!peraggstate->transtypeByVal &&
-	DatumGetPointer(newVal) != DatumGetPointer(pergroupstate->transValue))
+		DatumGetPointer(newVal) != DatumGetPointer(pergroupstate->transValue))
 	{
 		if (!fcinfo.isnull)
 		{
@@ -465,24 +469,24 @@ process_sorted_aggregate(AggState *aggstate,
 	tuplesort_performsort(peraggstate->sortstate);
 
 	/*
-	 * Note: if input type is pass-by-ref, the datums returned by the sort
-	 * are freshly palloc'd in the per-query context, so we must be
-	 * careful to pfree them when they are no longer needed.
+	 * Note: if input type is pass-by-ref, the datums returned by the sort are
+	 * freshly palloc'd in the per-query context, so we must be careful to
+	 * pfree them when they are no longer needed.
 	 */
 
 	while (tuplesort_getdatum(peraggstate->sortstate, true,
 							  &newVal, &isNull))
 	{
 		/*
-		 * DISTINCT always suppresses nulls, per SQL spec, regardless of
-		 * the transition function's strictness.
+		 * DISTINCT always suppresses nulls, per SQL spec, regardless of the
+		 * transition function's strictness.
 		 */
 		if (isNull)
 			continue;
 
 		/*
-		 * Clear and select the working context for evaluation of the
-		 * equality function and transition function.
+		 * Clear and select the working context for evaluation of the equality
+		 * function and transition function.
 		 */
 		MemoryContextReset(workcontext);
 		oldContext = MemoryContextSwitchTo(workcontext);
@@ -540,9 +544,8 @@ finalize_aggregate(AggState *aggstate,
 	{
 		FunctionCallInfoData fcinfo;
 
-		MemSet(&fcinfo, 0, sizeof(fcinfo));
-		fcinfo.flinfo = &peraggstate->finalfn;
-		fcinfo.nargs = 1;
+		InitFunctionCallInfoData(fcinfo, &(peraggstate->finalfn), 1,
+								 (void *) aggstate, NULL);
 		fcinfo.arg[0] = pergroupstate->transValue;
 		fcinfo.argnull[0] = pergroupstate->transValueIsNull;
 		if (fcinfo.flinfo->fn_strict && pergroupstate->transValueIsNull)
@@ -604,6 +607,26 @@ build_hash_table(AggState *aggstate)
 											  tmpmem);
 }
 
+/*
+ * Estimate per-hash-table-entry overhead for the planner.
+ *
+ * Note that the estimate does not include space for pass-by-reference
+ * transition data values.
+ */
+Size
+hash_agg_entry_size(int numAggs)
+{
+	Size		entrysize;
+
+	/* This must match build_hash_table */
+	entrysize = sizeof(AggHashEntryData) +
+		(numAggs - 1) *sizeof(AggStatePerGroupData);
+	/* Account for hashtable overhead */
+	entrysize += 2 * sizeof(void *);
+	entrysize = MAXALIGN(entrysize);
+	return entrysize;
+}
+
 /*
  * Find or create a hashtable entry for the tuple group containing the
  * given tuple.
@@ -699,8 +722,8 @@ agg_retrieve_direct(AggState *aggstate)
 	while (!aggstate->agg_done)
 	{
 		/*
-		 * If we don't already have the first tuple of the new group,
-		 * fetch it from the outer plan.
+		 * If we don't already have the first tuple of the new group, fetch it
+		 * from the outer plan.
 		 */
 		if (aggstate->grp_firstTuple == NULL)
 		{
@@ -708,10 +731,10 @@ agg_retrieve_direct(AggState *aggstate)
 			if (!TupIsNull(outerslot))
 			{
 				/*
-				 * Make a copy of the first input tuple; we will use this
-				 * for comparisons (in group mode) and for projection.
+				 * Make a copy of the first input tuple; we will use this for
+				 * comparisons (in group mode) and for projection.
 				 */
-				aggstate->grp_firstTuple = heap_copytuple(outerslot->val);
+				aggstate->grp_firstTuple = ExecCopySlotTuple(outerslot);
 			}
 			else
 			{
@@ -737,8 +760,8 @@ agg_retrieve_direct(AggState *aggstate)
 		{
 			/*
 			 * Store the copied first input tuple in the tuple table slot
-			 * reserved for it.  The tuple will be deleted when it is
-			 * cleared from the slot.
+			 * reserved for it.  The tuple will be deleted when it is cleared
+			 * from the slot.
 			 */
 			ExecStoreTuple(aggstate->grp_firstTuple,
 						   firstSlot,
@@ -776,17 +799,16 @@ agg_retrieve_direct(AggState *aggstate)
 				 */
 				if (node->aggstrategy == AGG_SORTED)
 				{
-					if (!execTuplesMatch(firstSlot->val,
-										 outerslot->val,
-										 firstSlot->ttc_tupleDescriptor,
+					if (!execTuplesMatch(firstSlot,
+										 outerslot,
 										 node->numCols, node->grpColIdx,
 										 aggstate->eqfunctions,
-									  tmpcontext->ecxt_per_tuple_memory))
+										 tmpcontext->ecxt_per_tuple_memory))
 					{
 						/*
 						 * Save the first input tuple of the next group.
 						 */
-						aggstate->grp_firstTuple = heap_copytuple(outerslot->val);
+						aggstate->grp_firstTuple = ExecCopySlotTuple(outerslot);
 						break;
 					}
 				}
@@ -813,44 +835,23 @@ agg_retrieve_direct(AggState *aggstate)
 		 * If we have no first tuple (ie, the outerPlan didn't return
 		 * anything), create a dummy all-nulls input tuple for use by
 		 * ExecQual/ExecProject. 99.44% of the time this is a waste of cycles,
-		 * because ordinarily the projected output tuple's targetlist
-		 * cannot contain any direct (non-aggregated) references to input
-		 * columns, so the dummy tuple will not be referenced. However
-		 * there are special cases where this isn't so --- in particular
-		 * an UPDATE involving an aggregate will have a targetlist
-		 * reference to ctid.  We need to return a null for ctid in that
-		 * situation, not coredump.
+		 * because ordinarily the projected output tuple's targetlist cannot
+		 * contain any direct (non-aggregated) references to input columns, so
+		 * the dummy tuple will not be referenced. However there are special
+		 * cases where this isn't so --- in particular an UPDATE involving an
+		 * aggregate will have a targetlist reference to ctid.	We need to
+		 * return a null for ctid in that situation, not coredump.
 		 *
 		 * The values returned for the aggregates will be the initial values
 		 * of the transition functions.
 		 */
 		if (TupIsNull(firstSlot))
 		{
-			TupleDesc	tupType;
-
 			/* Should only happen in non-grouped mode */
 			Assert(node->aggstrategy == AGG_PLAIN);
 			Assert(aggstate->agg_done);
 
-			tupType = firstSlot->ttc_tupleDescriptor;
-			/* watch out for zero-column input tuples, though... */
-			if (tupType && tupType->natts > 0)
-			{
-				HeapTuple	nullsTuple;
-				Datum	   *dvalues;
-				char	   *dnulls;
-
-				dvalues = (Datum *) palloc0(sizeof(Datum) * tupType->natts);
-				dnulls = (char *) palloc(sizeof(char) * tupType->natts);
-				MemSet(dnulls, 'n', sizeof(char) * tupType->natts);
-				nullsTuple = heap_formtuple(tupType, dvalues, dnulls);
-				ExecStoreTuple(nullsTuple,
-							   firstSlot,
-							   InvalidBuffer,
-							   true);
-				pfree(dvalues);
-				pfree(dnulls);
-			}
+			ExecStoreAllNullTuple(firstSlot);
 		}
 
 		/*
@@ -860,14 +861,14 @@ agg_retrieve_direct(AggState *aggstate)
 		econtext->ecxt_scantuple = firstSlot;
 
 		/*
-		 * Check the qual (HAVING clause); if the group does not match,
-		 * ignore it and loop back to try to process another group.
+		 * Check the qual (HAVING clause); if the group does not match, ignore
+		 * it and loop back to try to process another group.
 		 */
 		if (ExecQual(aggstate->ss.ps.qual, econtext, false))
 		{
 			/*
 			 * Form and return a projection tuple using the aggregate results
-			 * and the representative input tuple.  Note we do not support
+			 * and the representative input tuple.	Note we do not support
 			 * aggregates returning sets ...
 			 */
 			return ExecProject(projInfo, NULL);
@@ -897,8 +898,8 @@ agg_fill_hash_table(AggState *aggstate)
 	tmpcontext = aggstate->tmpcontext;
 
 	/*
-	 * Process each outer-plan tuple, and then fetch the next one, until
-	 * we exhaust the outer plan.
+	 * Process each outer-plan tuple, and then fetch the next one, until we
+	 * exhaust the outer plan.
 	 */
 	for (;;)
 	{
@@ -973,8 +974,8 @@ agg_retrieve_hash_table(AggState *aggstate)
 		ResetExprContext(econtext);
 
 		/*
-		 * Store the copied first input tuple in the tuple table slot
-		 * reserved for it, so that it can be used in ExecProject.
+		 * Store the copied first input tuple in the tuple table slot reserved
+		 * for it, so that it can be used in ExecProject.
 		 */
 		ExecStoreTuple(entry->shared.firstTuple,
 					   firstSlot,
@@ -1004,14 +1005,14 @@ agg_retrieve_hash_table(AggState *aggstate)
 		econtext->ecxt_scantuple = firstSlot;
 
 		/*
-		 * Check the qual (HAVING clause); if the group does not match,
-		 * ignore it and loop back to try to process another group.
+		 * Check the qual (HAVING clause); if the group does not match, ignore
+		 * it and loop back to try to process another group.
 		 */
 		if (ExecQual(aggstate->ss.ps.qual, econtext, false))
 		{
 			/*
 			 * Form and return a projection tuple using the aggregate results
-			 * and the representative input tuple.  Note we do not support
+			 * and the representative input tuple.	Note we do not support
 			 * aggregates returning sets ...
 			 */
 			return ExecProject(projInfo, NULL);
@@ -1038,7 +1039,7 @@ ExecInitAgg(Agg *node, EState *estate)
 	ExprContext *econtext;
 	int			numaggs,
 				aggno;
-	List	   *alist;
+	ListCell   *l;
 
 	/*
 	 * create state structure
@@ -1059,8 +1060,8 @@ ExecInitAgg(Agg *node, EState *estate)
 
 	/*
 	 * Create expression contexts.	We need two, one for per-input-tuple
-	 * processing and one for per-output-tuple processing.	We cheat a
-	 * little by using ExecAssignExprContext() to build both.
+	 * processing and one for per-output-tuple processing.	We cheat a little
+	 * by using ExecAssignExprContext() to build both.
 	 */
 	ExecAssignExprContext(estate, &aggstate->ss.ps);
 	aggstate->tmpcontext = aggstate->ss.ps.ps_ExprContext;
@@ -1068,10 +1069,10 @@ ExecInitAgg(Agg *node, EState *estate)
 
 	/*
 	 * We also need a long-lived memory context for holding hashtable data
-	 * structures and transition values.  NOTE: the details of what is
-	 * stored in aggcontext and what is stored in the regular per-query
-	 * memory context are driven by a simple decision: we want to reset
-	 * the aggcontext in ExecReScanAgg to recover no-longer-wanted space.
+	 * structures and transition values.  NOTE: the details of what is stored
+	 * in aggcontext and what is stored in the regular per-query memory
+	 * context are driven by a simple decision: we want to reset the
+	 * aggcontext in ExecReScanAgg to recover no-longer-wanted space.
 	 */
 	aggstate->aggcontext =
 		AllocSetContextCreate(CurrentMemoryContext,
@@ -1092,10 +1093,10 @@ ExecInitAgg(Agg *node, EState *estate)
 	 * initialize child expressions
 	 *
 	 * Note: ExecInitExpr finds Aggrefs for us, and also checks that no aggs
-	 * contain other agg calls in their arguments.	This would make no
-	 * sense under SQL semantics anyway (and it's forbidden by the spec).
-	 * Because that is true, we don't need to worry about evaluating the
-	 * aggs in any particular order.
+	 * contain other agg calls in their arguments.	This would make no sense
+	 * under SQL semantics anyway (and it's forbidden by the spec). Because
+	 * that is true, we don't need to worry about evaluating the aggs in any
+	 * particular order.
 	 */
 	aggstate->ss.ps.targetlist = (List *)
 		ExecInitExpr((Expr *) node->plan.targetlist,
@@ -1119,30 +1120,29 @@ ExecInitAgg(Agg *node, EState *estate)
 	 * Initialize result tuple type and projection info.
 	 */
 	ExecAssignResultTypeFromTL(&aggstate->ss.ps);
-	ExecAssignProjectionInfo(&aggstate->ss.ps);
+	ExecAssignProjectionInfo(&aggstate->ss.ps, NULL);
 
 	/*
 	 * get the count of aggregates in targetlist and quals
 	 */
 	numaggs = aggstate->numaggs;
-	Assert(numaggs == length(aggstate->aggs));
+	Assert(numaggs == list_length(aggstate->aggs));
 	if (numaggs <= 0)
 	{
 		/*
-		 * This is not an error condition: we might be using the Agg node
-		 * just to do hash-based grouping.	Even in the regular case,
-		 * constant-expression simplification could optimize away all of
-		 * the Aggrefs in the targetlist and qual.	So keep going, but
-		 * force local copy of numaggs positive so that palloc()s below
-		 * don't choke.
+		 * This is not an error condition: we might be using the Agg node just
+		 * to do hash-based grouping.  Even in the regular case,
+		 * constant-expression simplification could optimize away all of the
+		 * Aggrefs in the targetlist and qual.	So keep going, but force local
+		 * copy of numaggs positive so that palloc()s below don't choke.
 		 */
 		numaggs = 1;
 	}
 
 	/*
-	 * If we are grouping, precompute fmgr lookup data for inner loop. We
-	 * need both equality and hashing functions to do it by hashing, but
-	 * only equality if not hashing.
+	 * If we are grouping, precompute fmgr lookup data for inner loop. We need
+	 * both equality and hashing functions to do it by hashing, but only
+	 * equality if not hashing.
 	 */
 	if (node->numCols > 0)
 	{
@@ -1160,8 +1160,8 @@ ExecInitAgg(Agg *node, EState *estate)
 	}
 
 	/*
-	 * Set up aggregate-result storage in the output expr context, and
-	 * also allocate my private per-agg working storage
+	 * Set up aggregate-result storage in the output expr context, and also
+	 * allocate my private per-agg working storage
 	 */
 	econtext = aggstate->ss.ps.ps_ExprContext;
 	econtext->ecxt_aggvalues = (Datum *) palloc0(sizeof(Datum) * numaggs);
@@ -1186,15 +1186,15 @@ ExecInitAgg(Agg *node, EState *estate)
 	/*
 	 * Perform lookups of aggregate function info, and initialize the
 	 * unchanging fields of the per-agg data.  We also detect duplicate
-	 * aggregates (for example, "SELECT sum(x) ... HAVING sum(x) > 0").
-	 * When duplicates are detected, we only make an AggStatePerAgg struct
-	 * for the first one.  The clones are simply pointed at the same
-	 * result entry by giving them duplicate aggno values.
+	 * aggregates (for example, "SELECT sum(x) ... HAVING sum(x) > 0"). When
+	 * duplicates are detected, we only make an AggStatePerAgg struct for the
+	 * first one.  The clones are simply pointed at the same result entry by
+	 * giving them duplicate aggno values.
 	 */
 	aggno = -1;
-	foreach(alist, aggstate->aggs)
+	foreach(l, aggstate->aggs)
 	{
-		AggrefExprState *aggrefstate = (AggrefExprState *) lfirst(alist);
+		AggrefExprState *aggrefstate = (AggrefExprState *) lfirst(l);
 		Aggref	   *aggref = (Aggref *) aggrefstate->xprstate.expr;
 		AggStatePerAgg peraggstate;
 		Oid			inputType;
@@ -1237,9 +1237,9 @@ ExecInitAgg(Agg *node, EState *estate)
 		peraggstate->aggref = aggref;
 
 		/*
-		 * Get actual datatype of the input.  We need this because it may
-		 * be different from the agg's declared input type, when the agg
-		 * accepts ANY (eg, COUNT(*)) or ANYARRAY or ANYELEMENT.
+		 * Get actual datatype of the input.  We need this because it may be
+		 * different from the agg's declared input type, when the agg accepts
+		 * ANY (eg, COUNT(*)) or ANYARRAY or ANYELEMENT.
 		 */
 		inputType = exprType((Node *) aggref->target);
 
@@ -1264,7 +1264,7 @@ ExecInitAgg(Agg *node, EState *estate)
 		/* Check that aggregate owner has permission to call component fns */
 		{
 			HeapTuple	procTuple;
-			AclId		aggOwner;
+			Oid			aggOwner;
 
 			procTuple = SearchSysCache(PROCOID,
 									   ObjectIdGetDatum(aggref->aggfnoid),
@@ -1295,15 +1295,16 @@ ExecInitAgg(Agg *node, EState *estate)
 		if (aggtranstype == ANYARRAYOID || aggtranstype == ANYELEMENTOID)
 		{
 			/* have to fetch the agg's declared input type... */
-			Oid			agg_arg_types[FUNC_MAX_ARGS];
+			Oid		   *agg_arg_types;
 			int			agg_nargs;
 
 			(void) get_func_signature(aggref->aggfnoid,
-									  agg_arg_types, &agg_nargs);
+									  &agg_arg_types, &agg_nargs);
 			Assert(agg_nargs == 1);
 			aggtranstype = resolve_generic_type(aggtranstype,
 												inputType,
 												agg_arg_types[0]);
+			pfree(agg_arg_types);
 		}
 
 		/* build expression trees using actual argument & result types */
@@ -1332,8 +1333,8 @@ ExecInitAgg(Agg *node, EState *estate)
 						&peraggstate->transtypeByVal);
 
 		/*
-		 * initval is potentially null, so don't try to access it as a
-		 * struct field. Must do it the hard way with SysCacheGetAttr.
+		 * initval is potentially null, so don't try to access it as a struct
+		 * field. Must do it the hard way with SysCacheGetAttr.
 		 */
 		textInitVal = SysCacheGetAttr(AGGFNOID, aggTuple,
 									  Anum_pg_aggregate_agginitval,
@@ -1346,11 +1347,11 @@ ExecInitAgg(Agg *node, EState *estate)
 												   aggtranstype);
 
 		/*
-		 * If the transfn is strict and the initval is NULL, make sure
-		 * input type and transtype are the same (or at least binary-
-		 * compatible), so that it's OK to use the first input value as
-		 * the initial transValue.	This should have been checked at agg
-		 * definition time, but just in case...
+		 * If the transfn is strict and the initval is NULL, make sure input
+		 * type and transtype are the same (or at least binary- compatible),
+		 * so that it's OK to use the first input value as the initial
+		 * transValue.	This should have been checked at agg definition time,
+		 * but just in case...
 		 */
 		if (peraggstate->transfn.fn_strict && peraggstate->initValueIsNull)
 		{
@@ -1391,29 +1392,17 @@ ExecInitAgg(Agg *node, EState *estate)
 static Datum
 GetAggInitVal(Datum textInitVal, Oid transtype)
 {
-	char	   *strInitVal;
-	HeapTuple	tup;
 	Oid			typinput,
-				typelem;
+				typioparam;
+	char	   *strInitVal;
 	Datum		initVal;
 
+	getTypeInputInfo(transtype, &typinput, &typioparam);
 	strInitVal = DatumGetCString(DirectFunctionCall1(textout, textInitVal));
-
-	tup = SearchSysCache(TYPEOID,
-						 ObjectIdGetDatum(transtype),
-						 0, 0, 0);
-	if (!HeapTupleIsValid(tup))
-		elog(ERROR, "cache lookup failed for type %u", transtype);
-
-	typinput = ((Form_pg_type) GETSTRUCT(tup))->typinput;
-	typelem = ((Form_pg_type) GETSTRUCT(tup))->typelem;
-	ReleaseSysCache(tup);
-
 	initVal = OidFunctionCall3(typinput,
 							   CStringGetDatum(strInitVal),
-							   ObjectIdGetDatum(typelem),
+							   ObjectIdGetDatum(typioparam),
 							   Int32GetDatum(-1));
-
 	pfree(strInitVal);
 	return initVal;
 }
@@ -1468,18 +1457,18 @@ ExecReScanAgg(AggState *node, ExprContext *exprCtxt)
 	if (((Agg *) node->ss.ps.plan)->aggstrategy == AGG_HASHED)
 	{
 		/*
-		 * In the hashed case, if we haven't yet built the hash table then
-		 * we can just return; nothing done yet, so nothing to undo. If
-		 * subnode's chgParam is not NULL then it will be re-scanned by
-		 * ExecProcNode, else no reason to re-scan it at all.
+		 * In the hashed case, if we haven't yet built the hash table then we
+		 * can just return; nothing done yet, so nothing to undo. If subnode's
+		 * chgParam is not NULL then it will be re-scanned by ExecProcNode,
+		 * else no reason to re-scan it at all.
 		 */
 		if (!node->table_filled)
 			return;
 
 		/*
 		 * If we do have the hash table and the subplan does not have any
-		 * parameter changes, then we can just rescan the existing hash
-		 * table; no need to build it again.
+		 * parameter changes, then we can just rescan the existing hash table;
+		 * no need to build it again.
 		 */
 		if (((PlanState *) node)->lefttree->chgParam == NULL)
 		{
@@ -1520,7 +1509,9 @@ ExecReScanAgg(AggState *node, ExprContext *exprCtxt)
 	}
 	else
 	{
-		/* Reset the per-group state (in particular, mark transvalues null) */
+		/*
+		 * Reset the per-group state (in particular, mark transvalues null)
+		 */
 		MemSet(node->pergroup, 0,
 			   sizeof(AggStatePerGroupData) * node->numaggs);
 	}
