@@ -9,7 +9,7 @@
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/optimizer/util/plancat.c,v 1.100 2004/12/31 22:00:23 pgsql Exp $
+ *	  $PostgreSQL: pgsql/src/backend/optimizer/util/plancat.c,v 1.114.2.1 2005/11/22 18:23:12 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -19,15 +19,17 @@
 
 #include "access/genam.h"
 #include "access/heapam.h"
-#include "catalog/catname.h"
 #include "catalog/pg_amop.h"
 #include "catalog/pg_inherits.h"
 #include "catalog/pg_index.h"
 #include "nodes/makefuncs.h"
 #include "optimizer/clauses.h"
 #include "optimizer/plancat.h"
+#include "optimizer/prep.h"
 #include "optimizer/tlist.h"
 #include "parser/parsetree.h"
+#include "parser/parse_expr.h"
+#include "parser/parse_relation.h"
 #include "rewrite/rewriteManip.h"
 #include "utils/builtins.h"
 #include "utils/fmgroids.h"
@@ -39,7 +41,7 @@
 
 
 static void estimate_rel_size(Relation rel, int32 *attr_widths,
-							  BlockNumber *pages, double *tuples);
+				  BlockNumber *pages, double *tuples);
 
 
 /*
@@ -67,7 +69,25 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 	bool		hasindex;
 	List	   *indexinfos = NIL;
 
-	relation = heap_open(relationObjectId, AccessShareLock);
+	/*
+	 * Normally, we can assume the rewriter already acquired at least
+	 * AccessShareLock on each relation used in the query.	However this will
+	 * not be the case for relations added to the query because they are
+	 * inheritance children of some relation mentioned explicitly. For them,
+	 * this is the first access during the parse/rewrite/plan pipeline, and so
+	 * we need to obtain and keep a suitable lock.
+	 *
+	 * XXX really, a suitable lock is RowShareLock if the relation is an
+	 * UPDATE/DELETE target, and AccessShareLock otherwise.  However we cannot
+	 * easily tell here which to get, so for the moment just get
+	 * AccessShareLock always.	The executor will get the right lock when it
+	 * runs, which means there is a very small chance of deadlock trying to
+	 * upgrade our lock.
+	 */
+	if (rel->reloptkind == RELOPT_BASEREL)
+		relation = heap_open(relationObjectId, NoLock);
+	else
+		relation = heap_open(relationObjectId, AccessShareLock);
 
 	rel->min_attr = FirstLowInvalidHeapAttributeNumber + 1;
 	rel->max_attr = RelationGetNumberOfAttributes(relation);
@@ -85,8 +105,7 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 					  &rel->pages, &rel->tuples);
 
 	/*
-	 * Make list of indexes.  Ignore indexes on system catalogs if told
-	 * to.
+	 * Make list of indexes.  Ignore indexes on system catalogs if told to.
 	 */
 	if (IsIgnoringSystemIndexes() && IsSystemClass(relation->rd_rel))
 		hasindex = false;
@@ -124,11 +143,12 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 			info = makeNode(IndexOptInfo);
 
 			info->indexoid = index->indexrelid;
+			info->rel = rel;
 			info->ncolumns = ncolumns = index->indnatts;
 
 			/*
-			 * Need to make classlist and ordering arrays large enough to
-			 * put a terminating 0 at the end of each one.
+			 * Need to make classlist and ordering arrays large enough to put
+			 * a terminating 0 at the end of each one.
 			 */
 			info->indexkeys = (int *) palloc(sizeof(int) * ncolumns);
 			info->classlist = (Oid *) palloc0(sizeof(Oid) * (ncolumns + 1));
@@ -136,16 +156,16 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 
 			for (i = 0; i < ncolumns; i++)
 			{
-				info->classlist[i] = index->indclass[i];
-				info->indexkeys[i] = index->indkey[i];
+				info->classlist[i] = indexRelation->rd_indclass->values[i];
+				info->indexkeys[i] = index->indkey.values[i];
 			}
 
 			info->relam = indexRelation->rd_rel->relam;
-			info->amcostestimate = index_cost_estimator(indexRelation);
+			info->amcostestimate = indexRelation->rd_am->amcostestimate;
+			info->amoptionalkey = indexRelation->rd_am->amoptionalkey;
 
 			/*
-			 * Fetch the ordering operators associated with the index, if
-			 * any.
+			 * Fetch the ordering operators associated with the index, if any.
 			 */
 			amorderstrategy = indexRelation->rd_am->amorderstrategy;
 			if (amorderstrategy != 0)
@@ -162,8 +182,8 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 			/*
 			 * Fetch the index expressions and predicate, if any.  We must
 			 * modify the copies we obtain from the relcache to have the
-			 * correct varno for the parent relation, so that they match
-			 * up correctly against qual clauses.
+			 * correct varno for the parent relation, so that they match up
+			 * correctly against qual clauses.
 			 */
 			info->indexprs = RelationGetIndexExpressions(indexRelation);
 			info->indpred = RelationGetIndexPredicate(indexRelation);
@@ -175,11 +195,11 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 			info->unique = index->indisunique;
 
 			/*
-			 * Estimate the index size.  If it's not a partial index, we
-			 * lock the number-of-tuples estimate to equal the parent table;
-			 * if it is partial then we have to use the same methods as we
-			 * would for a table, except we can be sure that the index is
-			 * not larger than the table.
+			 * Estimate the index size.  If it's not a partial index, we lock
+			 * the number-of-tuples estimate to equal the parent table; if it
+			 * is partial then we have to use the same methods as we would for
+			 * a table, except we can be sure that the index is not larger
+			 * than the table.
 			 */
 			if (info->indpred == NIL)
 			{
@@ -194,10 +214,6 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 					info->tuples = rel->tuples;
 			}
 
-			/* initialize cached join info to empty */
-			info->outer_relids = NULL;
-			info->inner_paths = NIL;
-
 			index_close(indexRelation);
 
 			indexinfos = lcons(info, indexinfos);
@@ -208,8 +224,8 @@ get_relation_info(Oid relationObjectId, RelOptInfo *rel)
 
 	rel->indexlist = indexinfos;
 
-	/* XXX keep the lock here? */
-	heap_close(relation, AccessShareLock);
+	/* close rel, but keep lock if any */
+	heap_close(relation, NoLock);
 }
 
 /*
@@ -223,8 +239,8 @@ static void
 estimate_rel_size(Relation rel, int32 *attr_widths,
 				  BlockNumber *pages, double *tuples)
 {
-	BlockNumber	curpages;
-	BlockNumber	relpages;
+	BlockNumber curpages;
+	BlockNumber relpages;
 	double		reltuples;
 	double		density;
 
@@ -234,7 +250,32 @@ estimate_rel_size(Relation rel, int32 *attr_widths,
 		case RELKIND_INDEX:
 		case RELKIND_TOASTVALUE:
 			/* it has storage, ok to call the smgr */
-			*pages = curpages = RelationGetNumberOfBlocks(rel);
+			curpages = RelationGetNumberOfBlocks(rel);
+
+			/*
+			 * HACK: if the relation has never yet been vacuumed, use a
+			 * minimum estimate of 10 pages.  This emulates a desirable aspect
+			 * of pre-8.0 behavior, which is that we wouldn't assume a newly
+			 * created relation is really small, which saves us from making
+			 * really bad plans during initial data loading.  (The plans are
+			 * not wrong when they are made, but if they are cached and used
+			 * again after the table has grown a lot, they are bad.) It would
+			 * be better to force replanning if the table size has changed a
+			 * lot since the plan was made ... but we don't currently have any
+			 * infrastructure for redoing cached plans at all, so we have to
+			 * kluge things here instead.
+			 *
+			 * We approximate "never vacuumed" by "has relpages = 0", which
+			 * means this will also fire on genuinely empty relations.	Not
+			 * great, but fortunately that's a seldom-seen case in the real
+			 * world, and it shouldn't degrade the quality of the plan too
+			 * much anyway to err in this direction.
+			 */
+			if (curpages < 10 && rel->rd_rel->relpages == 0)
+				curpages = 10;
+
+			/* report estimated # pages */
+			*pages = curpages;
 			/* quick exit if rel is clearly empty */
 			if (curpages == 0)
 			{
@@ -244,6 +285,7 @@ estimate_rel_size(Relation rel, int32 *attr_widths,
 			/* coerce values in pg_class to more desirable types */
 			relpages = (BlockNumber) rel->rd_rel->relpages;
 			reltuples = (double) rel->rd_rel->reltuples;
+
 			/*
 			 * If it's an index, discount the metapage.  This is a kluge
 			 * because it assumes more than it ought to about index contents;
@@ -264,19 +306,19 @@ estimate_rel_size(Relation rel, int32 *attr_widths,
 				 * When we have no data because the relation was truncated,
 				 * estimate tuple width from attribute datatypes.  We assume
 				 * here that the pages are completely full, which is OK for
-				 * tables (since they've presumably not been VACUUMed yet)
-				 * but is probably an overestimate for indexes.  Fortunately
+				 * tables (since they've presumably not been VACUUMed yet) but
+				 * is probably an overestimate for indexes.  Fortunately
 				 * get_relation_info() can clamp the overestimate to the
 				 * parent table's size.
 				 *
 				 * Note: this code intentionally disregards alignment
-				 * considerations, because (a) that would be gilding the
-				 * lily considering how crude the estimate is, and (b)
-				 * it creates platform dependencies in the default plans
-				 * which are kind of a headache for regression testing.
+				 * considerations, because (a) that would be gilding the lily
+				 * considering how crude the estimate is, and (b) it creates
+				 * platform dependencies in the default plans which are kind
+				 * of a headache for regression testing.
 				 */
-				int32	tuple_width = 0;
-				int		i;
+				int32		tuple_width = 0;
+				int			i;
 
 				for (i = 1; i <= RelationGetNumberOfAttributes(rel); i++)
 				{
@@ -317,6 +359,85 @@ estimate_rel_size(Relation rel, int32 *attr_widths,
 	}
 }
 
+
+/*
+ * get_relation_constraints
+ *
+ * Retrieve the CHECK constraint expressions of the given relation.
+ *
+ * Returns a List (possibly empty) of constraint expressions.  Each one
+ * has been canonicalized, and its Vars are changed to have the varno
+ * indicated by rel->relid.  This allows the expressions to be easily
+ * compared to expressions taken from WHERE.
+ *
+ * Note: at present this is invoked at most once per relation per planner
+ * run, and in many cases it won't be invoked at all, so there seems no
+ * point in caching the data in RelOptInfo.
+ */
+List *
+get_relation_constraints(Oid relationObjectId, RelOptInfo *rel)
+{
+	List	   *result = NIL;
+	Index		varno = rel->relid;
+	Relation	relation;
+	TupleConstr *constr;
+
+	/*
+	 * We assume the relation has already been safely locked.
+	 */
+	relation = heap_open(relationObjectId, NoLock);
+
+	constr = relation->rd_att->constr;
+	if (constr != NULL)
+	{
+		int			num_check = constr->num_check;
+		int			i;
+
+		for (i = 0; i < num_check; i++)
+		{
+			Node	   *cexpr;
+
+			cexpr = stringToNode(constr->check[i].ccbin);
+
+			/*
+			 * Run each expression through const-simplification and
+			 * canonicalization.  This is not just an optimization, but is
+			 * necessary, because we will be comparing it to
+			 * similarly-processed qual clauses, and may fail to detect valid
+			 * matches without this.  This must match the processing done to
+			 * qual clauses in preprocess_expression()!  (We can skip the
+			 * stuff involving subqueries, however, since we don't allow any
+			 * in check constraints.)
+			 */
+			cexpr = eval_const_expressions(cexpr);
+
+			cexpr = (Node *) canonicalize_qual((Expr *) cexpr);
+
+			/*
+			 * Also mark any coercion format fields as "don't care", so that
+			 * we can match to both explicit and implicit coercions.
+			 */
+			set_coercionform_dontcare(cexpr);
+
+			/* Fix Vars to have the desired varno */
+			if (varno != 1)
+				ChangeVarNodes(cexpr, 1, varno, 0);
+
+			/*
+			 * Finally, convert to implicit-AND format (that is, a List) and
+			 * append the resulting item(s) to our output list.
+			 */
+			result = list_concat(result,
+								 make_ands_implicit((Expr *) cexpr));
+		}
+	}
+
+	heap_close(relation, NoLock);
+
+	return result;
+}
+
+
 /*
  * build_physical_tlist
  *
@@ -332,45 +453,115 @@ estimate_rel_size(Relation rel, int32 *attr_widths,
  * relation, but that data is not readily available to ExecTypeFromTL.
  * For now, we don't apply the physical-tlist optimization when there are
  * dropped cols.
+ *
+ * We also support building a "physical" tlist for subqueries and functions,
+ * since the same optimization can occur in SubqueryScan and FunctionScan
+ * nodes.
  */
 List *
-build_physical_tlist(Query *root, RelOptInfo *rel)
+build_physical_tlist(PlannerInfo *root, RelOptInfo *rel)
 {
-	Index		varno = rel->relid;
-	RangeTblEntry *rte = rt_fetch(varno, root->rtable);
-	Relation	relation;
 	List	   *tlist = NIL;
+	Index		varno = rel->relid;
+	RangeTblEntry *rte = rt_fetch(varno, root->parse->rtable);
+	Relation	relation;
+	Query	   *subquery;
+	Var		   *var;
+	ListCell   *l;
 	int			attrno,
 				numattrs;
+	List	   *colvars;
 
-	Assert(rte->rtekind == RTE_RELATION);
-
-	relation = heap_open(rte->relid, AccessShareLock);
-
-	numattrs = RelationGetNumberOfAttributes(relation);
-
-	for (attrno = 1; attrno <= numattrs; attrno++)
+	switch (rte->rtekind)
 	{
-		Form_pg_attribute att_tup = relation->rd_att->attrs[attrno - 1];
+		case RTE_RELATION:
+			/* Assume we already have adequate lock */
+			relation = heap_open(rte->relid, NoLock);
 
-		if (att_tup->attisdropped)
-		{
-			/* found a dropped col, so punt */
-			tlist = NIL;
+			numattrs = RelationGetNumberOfAttributes(relation);
+			for (attrno = 1; attrno <= numattrs; attrno++)
+			{
+				Form_pg_attribute att_tup = relation->rd_att->attrs[attrno - 1];
+
+				if (att_tup->attisdropped)
+				{
+					/* found a dropped col, so punt */
+					tlist = NIL;
+					break;
+				}
+
+				var = makeVar(varno,
+							  attrno,
+							  att_tup->atttypid,
+							  att_tup->atttypmod,
+							  0);
+
+				tlist = lappend(tlist,
+								makeTargetEntry((Expr *) var,
+												attrno,
+												NULL,
+												false));
+			}
+
+			heap_close(relation, NoLock);
 			break;
-		}
 
-		tlist = lappend(tlist,
-						create_tl_element(makeVar(varno,
-												  attrno,
-												  att_tup->atttypid,
-												  att_tup->atttypmod,
-												  0),
-										  attrno));
+		case RTE_SUBQUERY:
+			subquery = rte->subquery;
+			foreach(l, subquery->targetList)
+			{
+				TargetEntry *tle = (TargetEntry *) lfirst(l);
+
+				/*
+				 * A resjunk column of the subquery can be reflected as
+				 * resjunk in the physical tlist; we need not punt.
+				 */
+				var = makeVar(varno,
+							  tle->resno,
+							  exprType((Node *) tle->expr),
+							  exprTypmod((Node *) tle->expr),
+							  0);
+
+				tlist = lappend(tlist,
+								makeTargetEntry((Expr *) var,
+												tle->resno,
+												NULL,
+												tle->resjunk));
+			}
+			break;
+
+		case RTE_FUNCTION:
+			expandRTE(rte, varno, 0, true /* include dropped */ ,
+					  NULL, &colvars);
+			foreach(l, colvars)
+			{
+				var = (Var *) lfirst(l);
+
+				/*
+				 * A non-Var in expandRTE's output means a dropped column;
+				 * must punt.
+				 */
+				if (!IsA(var, Var))
+				{
+					tlist = NIL;
+					break;
+				}
+
+				tlist = lappend(tlist,
+								makeTargetEntry((Expr *) var,
+												var->varattno,
+												NULL,
+												false));
+			}
+			break;
+
+		default:
+			/* caller error */
+			elog(ERROR, "unsupported RTE kind %d in build_physical_tlist",
+				 (int) rte->rtekind);
+			break;
 	}
 
-	heap_close(relation, AccessShareLock);
-
 	return tlist;
 }
 
@@ -384,7 +575,7 @@ build_physical_tlist(Query *root, RelOptInfo *rel)
  * See clause_selectivity() for the meaning of the additional parameters.
  */
 Selectivity
-restriction_selectivity(Query *root,
+restriction_selectivity(PlannerInfo *root,
 						Oid operator,
 						List *args,
 						int varRelid)
@@ -419,7 +610,7 @@ restriction_selectivity(Query *root,
  * operator relation, by calling the function manager.
  */
 Selectivity
-join_selectivity(Query *root,
+join_selectivity(PlannerInfo *root,
 				 Oid operator,
 				 List *args,
 				 JoinType jointype)
@@ -478,7 +669,7 @@ find_inheritance_children(Oid inhparent)
 				Anum_pg_inherits_inhparent,
 				BTEqualStrategyNumber, F_OIDEQ,
 				ObjectIdGetDatum(inhparent));
-	relation = heap_openr(InheritsRelationName, AccessShareLock);
+	relation = heap_open(InheritsRelationId, AccessShareLock);
 	scan = heap_beginscan(relation, SnapshotNow, 1, key);
 	while ((inheritsTuple = heap_getnext(scan, ForwardScanDirection)) != NULL)
 	{
@@ -536,11 +727,11 @@ has_unique_index(RelOptInfo *rel, AttrNumber attno)
 		IndexOptInfo *index = (IndexOptInfo *) lfirst(ilist);
 
 		/*
-		 * Note: ignore partial indexes, since they don't allow us to
-		 * conclude that all attr values are distinct.	We don't take any
-		 * interest in expressional indexes either. Also, a multicolumn
-		 * unique index doesn't allow us to conclude that just the
-		 * specified attr is unique.
+		 * Note: ignore partial indexes, since they don't allow us to conclude
+		 * that all attr values are distinct.  We don't take any interest in
+		 * expressional indexes either. Also, a multicolumn unique index
+		 * doesn't allow us to conclude that just the specified attr is
+		 * unique.
 		 */
 		if (index->unique &&
 			index->ncolumns == 1 &&
