@@ -10,7 +10,7 @@
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/postmaster/autovacuum.c,v 1.5.2.5 2006/01/20 15:17:13 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/postmaster/autovacuum.c,v 1.5.2.7 2007/01/14 20:18:30 alvherre Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -45,6 +45,8 @@
 #include "utils/fmgroids.h"
 #include "utils/memutils.h"
 #include "utils/ps_status.h"
+#include "utils/lsyscache.h"
+#include "utils/rel.h"
 #include "utils/relcache.h"
 
 
@@ -108,6 +110,8 @@ static void test_rel_for_autovac(Oid relid, PgStat_StatTabEntry *tabentry,
 					 List **toast_table_ids);
 static void autovacuum_do_vac_analyze(List *relids, bool dovacuum,
 						  bool doanalyze, bool freeze);
+static void autovac_report_activity(VacuumStmt *vacstmt,
+						List *relids);
 
 
 /*
@@ -478,30 +482,19 @@ autovac_get_database_list(void)
 }
 
 /*
- * Process a whole database.  If it's a template database or is disallowing
- * connection by means of datallowconn=false, then issue a VACUUM FREEZE.
- * Else use a plain VACUUM.
+ * Return a palloc'ed copy of the pg_database entry for the given database.
+ * Note that no lock is retained on the entry whatsoever, so it may be stale by
+ * the time the caller inspects it.  This is sufficient for our purposes
+ * however.
  */
-static void
-process_whole_db(void)
+static Form_pg_database
+get_pg_database_entry(Oid dbid)
 {
-	Relation	dbRel;
+	Form_pg_database dbForm;
 	ScanKeyData entry[1];
+	Relation	dbRel;
 	SysScanDesc scan;
 	HeapTuple	tup;
-	Form_pg_database dbForm;
-	bool		freeze;
-
-	/* Start a transaction so our commands have one to play into. */
-	StartTransactionCommand();
-
-	 /* functions in indexes may want a snapshot set */
-	ActiveSnapshot = CopySnapshot(GetTransactionSnapshot());
-
-	/*
-	 * Clean up any dead statistics collector entries for this DB.
-	 */
-	pgstat_vacuum_tabstat();
 
 	dbRel = heap_open(DatabaseRelationId, AccessShareLock);
 
@@ -519,17 +512,44 @@ process_whole_db(void)
 	if (!HeapTupleIsValid(tup))
 		elog(ERROR, "could not find tuple for database %u", MyDatabaseId);
 
-	dbForm = (Form_pg_database) GETSTRUCT(tup);
-
-	if (!dbForm->datallowconn || dbForm->datistemplate)
-		freeze = true;
-	else
-		freeze = false;
+	dbForm = (Form_pg_database) palloc(sizeof(FormData_pg_database));
+	memcpy(dbForm, GETSTRUCT(tup), sizeof(FormData_pg_database));
 
 	systable_endscan(scan);
 
 	heap_close(dbRel, AccessShareLock);
 
+	return dbForm;
+}
+
+/*
+ * Process a whole database.  If it's a template database or is disallowing
+ * connection by means of datallowconn=false, then issue a VACUUM FREEZE.
+ * Else use a plain VACUUM.
+ */
+static void
+process_whole_db(void)
+{
+	Form_pg_database dbForm;
+	bool		freeze;
+
+	/* Start a transaction so our commands have one to play into. */
+	StartTransactionCommand();
+
+	 /* functions in indexes may want a snapshot set */
+	ActiveSnapshot = CopySnapshot(GetTransactionSnapshot());
+
+	/*
+	 * Clean up any dead statistics collector entries for this DB.
+	 */
+	pgstat_vacuum_tabstat();
+
+	dbForm = get_pg_database_entry(MyDatabaseId);
+
+	freeze = (!dbForm->datallowconn || dbForm->datistemplate);
+
+	pfree(dbForm);
+
 	elog(DEBUG2, "autovacuum: VACUUM%s whole database",
 		 (freeze) ? " FREEZE" : "");
 
@@ -560,6 +580,8 @@ do_autovacuum(PgStat_StatDBEntry *dbentry)
 	List	   *toast_table_ids = NIL;
 	ListCell   *cell;
 	PgStat_StatDBEntry *shared;
+	Form_pg_database dbForm;
+	bool		istemplate;
 
 	/* Start a transaction so our commands have one to play into. */
 	StartTransactionCommand();
@@ -574,6 +596,14 @@ do_autovacuum(PgStat_StatDBEntry *dbentry)
 	 */
 	pgstat_vacuum_tabstat();
 
+	/*
+	 * In a template database, we need to avoid putting our Xid in any table,
+	 * so disallow analyzes and force use of VACUUM FREEZE.
+	 */
+	dbForm = get_pg_database_entry(MyDatabaseId);
+	istemplate = (!dbForm->datallowconn || dbForm->datistemplate);
+	pfree(dbForm);
+
 	/*
 	 * StartTransactionCommand and CommitTransactionCommand will automatically
 	 * switch to other contexts.  We need this one to keep the list of
@@ -690,10 +720,11 @@ do_autovacuum(PgStat_StatDBEntry *dbentry)
 		VacuumCostDelay = tab->vacuum_cost_delay;
 		VacuumCostLimit = tab->vacuum_cost_limit;
 
+		/* in a template database, we never analyze and force freezing */
 		autovacuum_do_vac_analyze(list_make1_oid(tab->relid),
 								  tab->dovacuum,
-								  tab->doanalyze,
-								  false);
+								  !istemplate && tab->doanalyze,
+								  istemplate);
 	}
 
 	/* Finally close out the last transaction. */
@@ -916,12 +947,75 @@ autovacuum_do_vac_analyze(List *relids, bool dovacuum, bool doanalyze,
 	vacstmt->relation = NULL;	/* all tables, or not used if relids != NIL */
 	vacstmt->va_cols = NIL;
 
+	/* Let pgstat know what we're doing */
+	autovac_report_activity(vacstmt, relids);
+
 	vacuum(vacstmt, relids);
 
 	pfree(vacstmt);
 	MemoryContextSwitchTo(old_cxt);
 }
 
+/*
+ * autovac_report_activity
+ * 		Report to pgstat what autovacuum is doing
+ *
+ * We send a SQL string corresponding to what the user would see if the
+ * equivalent command was to be issued manually.
+ *
+ * Note we assume that we are going to report the next command as soon as we're
+ * done with the current one, and exiting right after the last one, so we don't
+ * bother to report "<IDLE>" or some such.
+ */
+#define MAX_AUTOVAC_ACTIV_LEN (NAMEDATALEN * 2 + 32)
+static void
+autovac_report_activity(VacuumStmt *vacstmt, List *relids)
+{
+	char		activity[MAX_AUTOVAC_ACTIV_LEN];
+
+	/*
+	 * This case is not currently exercised by the autovac code.  Fill it in
+	 * if needed.
+	 */
+	if (list_length(relids) > 1)
+		elog(WARNING, "vacuuming >1 rel unsupported");
+
+	/* Report the command and possible options */
+	if (vacstmt->vacuum)
+		snprintf(activity, MAX_AUTOVAC_ACTIV_LEN,
+					   "VACUUM%s%s%s",
+					   vacstmt->full ? " FULL" : "",
+					   vacstmt->freeze ? " FREEZE" : "",
+					   vacstmt->analyze ? " ANALYZE" : "");
+	else if (vacstmt->analyze)
+		snprintf(activity, MAX_AUTOVAC_ACTIV_LEN,
+					   "ANALYZE");
+
+	/* Report the qualified name of the first relation, if any */
+	if (list_length(relids) > 0)
+	{
+		Oid			relid = linitial_oid(relids);
+		Relation	rel;
+
+		rel = RelationIdGetRelation(relid);
+		if (rel == NULL)
+			elog(WARNING, "cache lookup failed for relation %u", relid);
+		else
+		{
+			char   *nspname = get_namespace_name(RelationGetNamespace(rel));
+			int		len = strlen(activity);
+
+			snprintf(activity + len, MAX_AUTOVAC_ACTIV_LEN - len,
+					 " %s.%s", nspname, RelationGetRelationName(rel));
+
+			pfree(nspname);
+			RelationClose(rel);
+		}
+	}
+
+	pgstat_report_activity(activity);
+}
+
 /*
  * AutoVacuumingActive
  *		Check GUC vars and report whether the autovacuum process should be
