@@ -4,10 +4,10 @@
  *		Declarations for execution of SQL-language functions.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: functions.h,v 1.18 2001/11/05 17:46:33 momjian Exp $
+ * $PostgreSQL: pgsql/src/include/executor/functions.h,v 1.26 2005/10/15 02:49:44 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -15,7 +15,13 @@
 #define FUNCTIONS_H
 
 #include "fmgr.h"
+#include "nodes/execnodes.h"
+
 
 extern Datum fmgr_sql(PG_FUNCTION_ARGS);
 
+extern bool check_sql_fn_retval(Oid func_id, Oid rettype,
+					List *queryTreeList,
+					JunkFilter **junkFilter);
+
 #endif   /* FUNCTIONS_H */
