@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/pgsql/src/interfaces/ecpg/ecpglib/data.c,v 1.20.2.2 2005/08/24 10:35:54 meskes Exp $ */
+/* $PostgreSQL: pgsql/src/interfaces/ecpg/ecpglib/data.c,v 1.29.2.2 2006/06/21 10:29:50 meskes Exp $ */
 
 #define POSTGRES_ECPG_INTERNAL
 #include "postgres_fe.h"
@@ -16,12 +16,16 @@
 #include "pgtypes_timestamp.h"
 #include "pgtypes_interval.h"
 
-static bool garbage_left(enum ARRAY_TYPE isarray, char *scan_length, enum COMPAT_MODE compat)
+static bool
+garbage_left(enum ARRAY_TYPE isarray, char *scan_length, enum COMPAT_MODE compat)
 {
-	/* INFORMIX allows for selecting a numeric into an int, the result is truncated */
-	if (isarray == ECPG_ARRAY_NONE && INFORMIX_MODE(compat) && *scan_length == '.') 
+	/*
+	 * INFORMIX allows for selecting a numeric into an int, the result is
+	 * truncated
+	 */
+	if (isarray == ECPG_ARRAY_NONE && INFORMIX_MODE(compat) && *scan_length == '.')
 		return false;
-	
+
 	if (isarray == ECPG_ARRAY_ARRAY && *scan_length != ',' && *scan_length != '}')
 		return true;
 
@@ -44,13 +48,13 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 	char	   *pval = (char *) PQgetvalue(results, act_tuple, act_field);
 	int			value_for_indicator = 0;
 
-	ECPGlog("ECPGget_data line %d: RESULT: %s offset: %ld array: %d\n", lineno, pval ? pval : "", offset, isarray);
+	ECPGlog("ECPGget_data line %d: RESULT: %s offset: %ld array: %s\n", lineno, pval ? pval : "", offset, isarray ? "Yes" : "No");
 
 	/* We will have to decode the value */
 
 	/*
-	 * check for null value and set indicator accordingly, i.e. -1 if NULL
-	 * and 0 if not
+	 * check for null value and set indicator accordingly, i.e. -1 if NULL and
+	 * 0 if not
 	 */
 	if (PQgetisnull(results, act_tuple, act_field))
 		value_for_indicator = -1;
@@ -81,20 +85,24 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 				if (force_indicator == false)
 				{
 					/*
-					 * Informix has an additional way to specify NULLs
-					 * note that this uses special values to denote NULL
+					 * Informix has an additional way to specify NULLs note
+					 * that this uses special values to denote NULL
 					 */
-					ECPGset_informix_null(type, var + offset * act_tuple);
+					ECPGset_noind_null(type, var + offset * act_tuple);
 				}
 				else
 				{
-					ECPGraise(lineno, ECPG_MISSING_INDICATOR, ECPG_SQLSTATE_NULL_VALUE_NO_INDICATOR_PARAMETER, NULL);
+					ECPGraise(lineno, ECPG_MISSING_INDICATOR,
+							  ECPG_SQLSTATE_NULL_VALUE_NO_INDICATOR_PARAMETER,
+							  NULL);
 					return (false);
 				}
 			}
 			break;
 		default:
-			ECPGraise(lineno, ECPG_UNSUPPORTED, ECPG_SQLSTATE_ECPG_INTERNAL_ERROR, ECPGtype_name(ind_type));
+			ECPGraise(lineno, ECPG_UNSUPPORTED,
+					  ECPG_SQLSTATE_ECPG_INTERNAL_ERROR,
+					  ECPGtype_name(ind_type));
 			return (false);
 			break;
 	}
@@ -106,9 +114,10 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 	/* let's check if it really is an array if it should be one */
 	if (isarray == ECPG_ARRAY_ARRAY)
 	{
-		if (*pval != '{')
+		if (!pval || *pval != '{')
 		{
-			ECPGraise(lineno, ECPG_DATA_NOT_ARRAY, ECPG_SQLSTATE_DATATYPE_MISMATCH, NULL);
+			ECPGraise(lineno, ECPG_DATA_NOT_ARRAY,
+					  ECPG_SQLSTATE_DATATYPE_MISMATCH, NULL);
 			return (false);
 		}
 
@@ -146,9 +155,11 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					res = strtol(pval, &scan_length, 10);
 					if (garbage_left(isarray, scan_length, compat))
 					{
-						ECPGraise(lineno, ECPG_INT_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+						ECPGraise(lineno, ECPG_INT_FORMAT,
+								  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 						return (false);
 					}
+					pval = scan_length;
 				}
 				else
 					res = 0L;
@@ -178,9 +189,11 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					ures = strtoul(pval, &scan_length, 10);
 					if (garbage_left(isarray, scan_length, compat))
 					{
-						ECPGraise(lineno, ECPG_UINT_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+						ECPGraise(lineno, ECPG_UINT_FORMAT,
+								  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 						return (false);
 					}
+					pval = scan_length;
 				}
 				else
 					ures = 0L;
@@ -213,6 +226,7 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 						ECPGraise(lineno, ECPG_INT_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 						return (false);
 					}
+					pval = scan_length;
 				}
 				else
 					*((long long int *) (var + offset * act_tuple)) = (long long) 0;
@@ -225,11 +239,12 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 				{
 					*((unsigned long long int *) (var + offset * act_tuple)) = strtoull(pval, &scan_length, 10);
 					if ((isarray && *scan_length != ',' && *scan_length != '}')
-						|| (!isarray && !(INFORMIX_MODE(compat) && *scan_length == '.') && *scan_length != '\0' && *scan_length != ' '))	/* Garbage left */
+						|| (!isarray && !(INFORMIX_MODE(compat) && *scan_length == '.') && *scan_length != '\0' && *scan_length != ' '))		/* Garbage left */
 					{
 						ECPGraise(lineno, ECPG_UINT_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 						return (false);
 					}
+					pval = scan_length;
 				}
 				else
 					*((unsigned long long int *) (var + offset * act_tuple)) = (long long) 0;
@@ -252,9 +267,11 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 
 					if (garbage_left(isarray, scan_length, compat))
 					{
-						ECPGraise(lineno, ECPG_FLOAT_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+						ECPGraise(lineno, ECPG_FLOAT_FORMAT,
+								  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 						return (false);
 					}
+					pval = scan_length;
 				}
 				else
 					dres = 0.0;
@@ -283,7 +300,9 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 						else if (offset == sizeof(int))
 							*((int *) (var + offset * act_tuple)) = false;
 						else
-							ECPGraise(lineno, ECPG_CONVERT_BOOL, ECPG_SQLSTATE_DATATYPE_MISMATCH, "different size");
+							ECPGraise(lineno, ECPG_CONVERT_BOOL,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH,
+									  "different size");
 						break;
 					}
 					else if (pval[0] == 't' && pval[1] == '\0')
@@ -293,7 +312,9 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 						else if (offset == sizeof(int))
 							*((int *) (var + offset * act_tuple)) = true;
 						else
-							ECPGraise(lineno, ECPG_CONVERT_BOOL, ECPG_SQLSTATE_DATATYPE_MISMATCH, "different size");
+							ECPGraise(lineno, ECPG_CONVERT_BOOL,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH,
+									  "different size");
 						break;
 					}
 					else if (pval[0] == '\0' && PQgetisnull(results, act_tuple, act_field))
@@ -303,14 +324,16 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					}
 				}
 
-				ECPGraise(lineno, ECPG_CONVERT_BOOL, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+				ECPGraise(lineno, ECPG_CONVERT_BOOL,
+						  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 				return (false);
 				break;
 
 			case ECPGt_char:
 			case ECPGt_unsigned_char:
+				if (pval)
 				{
-					if (varcharsize == 0)
+					if (varcharsize == 0 || varcharsize > strlen(pval))
 						strncpy((char *) ((long) var + offset * act_tuple), pval, strlen(pval) + 1);
 					else
 					{
@@ -345,10 +368,12 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 							sqlca->sqlwarn[0] = sqlca->sqlwarn[1] = 'W';
 						}
 					}
+					pval += strlen(pval);
 				}
 				break;
 
 			case ECPGt_varchar:
+				if (pval)
 				{
 					struct ECPGgeneric_varchar *variable =
 					(struct ECPGgeneric_varchar *) ((long) var + offset * act_tuple);
@@ -391,6 +416,7 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 							variable->len = varcharsize;
 						}
 					}
+					pval += strlen(pval);
 				}
 				break;
 
@@ -406,16 +432,29 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					/* did we get an error? */
 					if (errno != 0)
 					{
-						ECPGlog("ECPGget_data line %d: RESULT: %s errno %d\n", lineno, pval ? pval : "", errno);
+						ECPGlog("ECPGget_data line %d: RESULT: %s errno %d\n",
+								lineno, pval ? pval : "", errno);
 
 						if (INFORMIX_MODE(compat))
 						{
-							/* Informix wants its own NULL value here instead of an error */
-							ECPGset_informix_null(ECPGt_numeric, nres);
+							/*
+							 * Informix wants its own NULL value here instead
+							 * of an error
+							 */
+							nres = PGTYPESnumeric_new();
+							if (nres)
+								ECPGset_noind_null(ECPGt_numeric, nres);
+							else
+							{
+								ECPGraise(lineno, ECPG_OUT_OF_MEMORY,
+									ECPG_SQLSTATE_ECPG_OUT_OF_MEMORY, NULL);
+								return (false);
+							}
 						}
 						else
 						{
-							ECPGraise(lineno, ECPG_NUMERIC_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							ECPGraise(lineno, ECPG_NUMERIC_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
@@ -426,10 +465,13 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 
 						if (garbage_left(isarray, scan_length, compat))
 						{
-							ECPGraise(lineno, ECPG_NUMERIC_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							free(nres);
+							ECPGraise(lineno, ECPG_NUMERIC_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
+					pval = scan_length;
 				}
 				else
 					nres = PGTYPESnumeric_from_asc("0.0", &scan_length);
@@ -438,6 +480,8 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					PGTYPESnumeric_copy(nres, (numeric *) (var + offset * act_tuple));
 				else
 					PGTYPESnumeric_to_decimal(nres, (decimal *) (var + offset * act_tuple));
+
+				free(nres);
 				break;
 
 			case ECPGt_interval:
@@ -453,12 +497,20 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					{
 						if (INFORMIX_MODE(compat))
 						{
-							/* Informix wants its own NULL value here instead of an error */
-							ECPGset_informix_null(ECPGt_interval, ires);
+							/*
+							 * Informix wants its own NULL value here instead
+							 * of an error
+							 */
+							ires = (interval *) ECPGalloc(sizeof(interval), lineno);
+							if (!ires)
+								return (false);
+
+							ECPGset_noind_null(ECPGt_interval, ires);
 						}
 						else
 						{
-							ECPGraise(lineno, ECPG_INTERVAL_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							ECPGraise(lineno, ECPG_INTERVAL_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
@@ -469,15 +521,19 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 
 						if (garbage_left(isarray, scan_length, compat))
 						{
-							ECPGraise(lineno, ECPG_INTERVAL_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							free(ires);
+							ECPGraise(lineno, ECPG_INTERVAL_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
+					pval = scan_length;
 				}
 				else
 					ires = PGTYPESinterval_from_asc("0 seconds", NULL);
 
 				PGTYPESinterval_copy(ires, (interval *) (var + offset * act_tuple));
+				free(ires);
 				break;
 			case ECPGt_date:
 				if (pval)
@@ -492,12 +548,16 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					{
 						if (INFORMIX_MODE(compat))
 						{
-							/* Informix wants its own NULL value here instead of an error */
-							ECPGset_informix_null(ECPGt_date, &ddres);
+							/*
+							 * Informix wants its own NULL value here instead
+							 * of an error
+							 */
+							ECPGset_noind_null(ECPGt_date, &ddres);
 						}
 						else
 						{
-							ECPGraise(lineno, ECPG_DATE_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							ECPGraise(lineno, ECPG_DATE_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
@@ -508,12 +568,14 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 
 						if (garbage_left(isarray, scan_length, compat))
 						{
-							ECPGraise(lineno, ECPG_DATE_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							ECPGraise(lineno, ECPG_DATE_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
 
 					*((date *) (var + offset * act_tuple)) = ddres;
+					pval = scan_length;
 				}
 				break;
 
@@ -530,12 +592,16 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 					{
 						if (INFORMIX_MODE(compat))
 						{
-							/* Informix wants its own NULL value here instead of an error */
-							ECPGset_informix_null(ECPGt_timestamp, &tres);
+							/*
+							 * Informix wants its own NULL value here instead
+							 * of an error
+							 */
+							ECPGset_noind_null(ECPGt_timestamp, &tres);
 						}
 						else
 						{
-							ECPGraise(lineno, ECPG_TIMESTAMP_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							ECPGraise(lineno, ECPG_TIMESTAMP_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
@@ -546,17 +612,21 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 
 						if (garbage_left(isarray, scan_length, compat))
 						{
-							ECPGraise(lineno, ECPG_TIMESTAMP_FORMAT, ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
+							ECPGraise(lineno, ECPG_TIMESTAMP_FORMAT,
+									  ECPG_SQLSTATE_DATATYPE_MISMATCH, pval);
 							return (false);
 						}
 					}
 
 					*((timestamp *) (var + offset * act_tuple)) = tres;
+					pval = scan_length;
 				}
 				break;
 
 			default:
-				ECPGraise(lineno, ECPG_UNSUPPORTED, ECPG_SQLSTATE_ECPG_INTERNAL_ERROR, ECPGtype_name(type));
+				ECPGraise(lineno, ECPG_UNSUPPORTED,
+						  ECPG_SQLSTATE_ECPG_INTERNAL_ERROR,
+						  ECPGtype_name(type));
 				return (false);
 				break;
 		}
@@ -568,7 +638,7 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 			++act_tuple;
 
 			/* set pval to the next entry */
-			for (; string || (*pval != ',' && *pval != '}'); ++pval)
+			for (; string || (*pval != ',' && *pval != '}' && *pval != '\0'); ++pval)
 				if (*pval == '"')
 					string = string ? false : true;
 
@@ -590,7 +660,7 @@ ECPGget_data(const PGresult *results, int act_tuple, int act_field, int lineno,
 			if (*pval == ' ')
 				++pval;
 		}
-	} while ((isarray == ECPG_ARRAY_ARRAY && *pval != '}') || (isarray == ECPG_ARRAY_VECTOR && *pval != '\0'));
+	} while (*pval != '\0' && ((isarray == ECPG_ARRAY_ARRAY && *pval != '}') || isarray == ECPG_ARRAY_VECTOR));
 
 	return (true);
 }
